%% Specify directory and the number of images to be processed
clc
clear


% Copy into directory if not in the right one. 
% C:\NTNU_Data\Dropbox\Victoria\Skole\Master\Double Reflector - Copy\Pictures to play with

Test = 'N-W';
Dir = 'C:\NTNU_Data\Pictures\dist670not_levelled25\';
Reg = ' registered';
Dot = ' dot';

NumPhotos = 1130;  % Number of photos to be processed
NumNoHit = 2;  % Number of a photo in cd with no hits

%% Directory
cd(horzcat(Dir,Test,Reg));   % Change directory

%% Counters
Picturecount=0;
IsEmpty=0;

%% Open one of the images
% The m-by-n size of the image (the pixel heigth and width of the picture)
% is taken from one of the images to be able to add laser dot fractions
% in the loop of the picture processing.

Compositions=imread('Reg_1.jpg');
Composition=zeros(size(Compositions(:,:,1)));
data=zeros(NumPhotos,1,7, 'double');

%% Average intensity of the absorber plate
% An image from the same dataset as the Test being analyzed with no
% laserdot is used as a measure of the background intensity of
% the absorber plate as a base value for thresholding the laser dot. 
% This method requires an input value of one of the images with noe hits.
% It also requred the lighting conditions to be very stable to et good
% results.

% Another way of getting an automized procedure would be to calculate this 
% mean for every image in an iterative manner where the first threshold 
% value is set at a low value, and a binary image of the absorber plate is 
% created based on this value. And the wanted thresholding value is then
% calculated based on the mean intensity of og the absorber plate on each 

Im = imread(strcat('Reg_', num2str(NumNoHit),'.jpg'));
level = graythresh(Im);
bwabsorber = im2bw(Im,level);
bwabsorber = bwareaopen(bwabsorber,1000);   % removes objects smaller than 1000 pixels 
bwabsorber=imfill(bwabsorber,'holes'); % Fills in small holes in at the border
bwabsorber=imclearborder(bwabsorber);     % Clears the border by supressing border structures
figure(1); imshow(bwabsorber,'InitialMagnification','fit')

STATS = regionprops(bwabsorber,Im,'MeanIntensity');
AbsIntensity = STATS.MeanIntensity;
STATS = regionprops(bwabsorber,Im,'MaxIntensity');
AbsMaxInt = STATS.MaxIntensity;

%% Loop for processing images
tic
for i = 1:NumPhotos;
    
    cd(horzcat(Dir,Test,Reg)); 
    
    Im = imread(strcat('Reg_', num2str(i),'.jpg'));

    Picturecount=Picturecount+1;

    %% Segmenting the laser dot by threshholding
    % The level of the thresholding is calculated based on the mean
    % intensity value of the absorberplate from one image representing all 
    % the images. 
    
    thresh = AbsIntensity*2.5;  % 250% higher valu than the absorbeplate
    bw = im2bw(Im, thresh/255);
    %figure(1); imshow(bw,'InitialMagnification','fit')

    %% Greythresh:
    % As a way to cancle out everything around the absorber plate, we use the
    % automatically calculated threshold function to separate the absorber from
    % the rest of the picture.
    level = graythresh(Im);
    bwabsorber = im2bw(Im,level);
    % figure(1); imshow(bwabsorber,'InitialMagnification','fit')
    
    %% Removing small blobs
    % Morphology can assist segmentation. This function removes all objects
    % smaller than a number of pixels
    bwabsorber = bwareaopen(bwabsorber,1000);   % removes objects smaller than 1000 pixels 
    % figure(1); imshow(bwabsorber,'InitialMagnification','fit')
    %% Clean up image
    bwClear=imfill(bwabsorber,'holes'); % Fills in small holes in at the border
    bwClear=imclearborder(bwClear);     % Clears the border by supressing border structures
    %figure(1); imshow(bwClear,'InitialMagnification','fit')
    %% Add image to bwClean
    bwClear=im2double(bwClear);
    bw=bwClear.*bw;
    %figure(1); imshow(bw,'InitialMagnification','fit')
    %% Find remaining regions
    % This is a way to count all connected objects in a binary image to see how
    % many objects we are dealing with.
    [B,L] = bwboundaries(bw,'noholes'); % L is the image with all connected regions 
                                              % saved as a number (1,2,3 etc.)
    numRegions = max(L(:));
    % figure(1); imshow(label2rgb(L),'InitialMagnification','fit')
    
    
   %% Need to cancle out small bright objects that are not the laser dot
    STATS = regionprops(bw, 'Area');
    Areavector=[STATS.Area];
    AreaDetected = sum(Areavector);
    
        %% Detect the images with no laser dot
    if AreaDetected < 100;
        IsEmpty=IsEmpty+1;
        
    else
        data(i,1,5) = 1;        % Meaningless for data that has wrong input.
    %% Measure total area of the detected laser dot 
    STATS = regionprops(bw, 'Area');
    Areavector=[STATS.Area];
    Area = sum(Areavector);

    % Here the area should be saved in the row corrsponding to the number
    % the picture is
    data(i,1,3) = Area;

    %% Measure the Mean Intensity of the detected dot

    stats = regionprops(bw,Im,'MeanIntensity');
    MeanIntensity = stats.MeanIntensity;

    % Here the mean intensity should be saved in the row corrsponding to the number
    % the picture is 
    data(i,1,4) = MeanIntensity;
    
    %% Record the pixel value of the detected laser dot 
    stats = regionprops(bw,Im,'WeightedCentroid');
    [LaserCoord] = stats.WeightedCentroid;
    
    data(i,1,6) = LaserCoord(1);
    data(i,1,7) = LaserCoord(2);
   
    
    %% Add the intensity to LaserDot
    Im = im2double(Im);
    LaserDot=bw.*Im;
    figure(1); imshow(LaserDot,'InitialMagnification','fit'),title('LaserDot');
    
    cd(horzcat(Dir,Test,Dot));
    imwrite(LaserDot,strcat('Dot_',num2str(i),'.jpg'));
    %% Subtract the isolated laser dot to see what we are missing
    ImageRest=Im-LaserDot;
    %figure(1); imshow(ImageRest,'InitialMagnification','fit')
    %% Adding the dot to a new picture. 
    Composition = Composition+LaserDot;
        
    end
    
end
toc
%% Calculate number of hits
% To get an idea of efficiency

Hits=Picturecount-IsEmpty;
Efficiency_hits=(Picturecount-IsEmpty)/Picturecount;

figure(1);
surf(Composition)
shading('interp');
colormap('hot');
colorbar;
view(90,0);

%% Find and exclude the wrong data
% In this section, the wrongly collected data will be removed. By wrong data, I mean
% the data that is collected that is not the red dots ont the absorber,
% but bright spots that 


% A better solution would be to design a better detection
% be to not detect this data (bright spots on the absorbe plate), but the
% time constraints made it difficult to come up with a better code

bwComposition=Composition>0;
CC = bwconncomp(bwComposition);
[B,L] = bwboundaries(bwComposition,'noholes');


STATSmean = regionprops(L,Composition,'MeanIntensity');
Mean = [STATSmean.MeanIntensity];

STATSmax = regionprops(L,Composition,'MaxIntensity');
Max = [STATSmax.MaxIntensity];
    
STATS = regionprops(L,'Area');
Areavector=[STATS.Area];

STATSmin = regionprops(L,Composition,'MinIntensity');
Min = [STATSmin.MinIntensity];

% Need a way to find out whether or not there is wrong data in the result.
% It can be that there is not, so before this part of the script is used,
% this should be investigated.

%% Remove wrongly detected spots from variable Composition
% this is a so called "quick fix" to the roblem of detecting the 

[maxArea,A] = max(Areavector(:));
[maxInt,Int] = max(Max(:));


while A~=Int;
    
    Exclude = true(size(bwComposition));
    Exclude(CC.PixelIdxList{Int}) = false;
    bwExcluded = im2double(Exclude);
    
    % Claculate the new composition matrix, and the properties to evaluate 
    % the while loop

    Composition = bwExcluded.*Composition;
    
    bwComposition=Composition>0;
    CC = bwconncomp(bwComposition);
    [B,L] = bwboundaries(bwComposition,'noholes');
    
    STATSmax = regionprops(L,Composition,'MaxIntensity');
    Max = [STATSmax.MaxIntensity];
    
    STATS = regionprops(L,'Area');
    Areavector=[STATS.Area];
    
    [maxArea,A] = max(Areavector(:))
    [maxInt,Int] = max(Max(:))
end

figure(1);
surf(Composition)
shading('interp');
colormap('hot');
colorbar;
view(90,0);

%% If this does not work, remove data in 3-by-3 matrices around the maximum
% This assures that no part of the right data is lost.

%MeanValue = Mean(A);
%critpoint = MeanValue*10;       % NB! This needs to match the graph
%[maxPoint,P] = max(Composition(:));
%Remove = 4;     % How many pixels to remove in each direction around maximum.
                % In a square matrix.

%[ii,jj] = ind2sub(size(Composition),P);

%while maxPoint > critpoint
%    [ii,jj] = ind2sub(size(Composition),P);
%    Composition([ii-Remove:ii+Remove],[jj-Remove:jj+Remove]) = 0;
%    [maxPoint,P] = max(Composition(:));
%end

%figure;
%surf(Composition)
%shading('interp');
%colormap('hot');
%colorbar;
%view(90,0);

%% Total area of the laser dot

Area = sum(Areavector);

%% Extract the weighted centroid of the composition of laser dots
STATS = regionprops(bwComposition,Composition,'WeightedCentroid');
Centroid = STATS.WeightedCentroid;

%% Convert composition to uint8, and calculate the real, mean pixel values
Composition8 = zeros(size(Composition),'uint8');
Composition8 = (Composition.*255)./Hits;

%% Find mean intensity of all the laser dots together
STATS = regionprops(bwComposition,Composition8,'MeanIntensity');
MeanIntensity = STATS.MeanIntensity;

%% Find minimum intensity of all the laser dots together
STATS = regionprops(bwComposition,Composition8,'MinIntensity');
MinIntensity = STATS.MinIntensity;

%% Max intensity of all the laser dots together
MaxIntensity = max(Composition8(:));

%% Locate the center of the absorber plate
cd(horzcat(Dir,Test,Reg));
Im = imread(horzcat('Reg_',num2str(NumNoHit),'.jpg'));
level = graythresh(Im);
bwabsorber = im2bw(Im,level);
bwabsorber = bwareaopen(bwabsorber,1000);   % removes objects smaller than 1000 pixels 
bwabsorber=imfill(bwabsorber,'holes'); % Fills in small holes in at the border
bwabsorber=imclearborder(bwabsorber);     % Clears the border by supressing border structures
STATS = regionprops(bwabsorber,'Centroid');
Centroid=STATS(1);      % First value x-value (horizontal), second value y-value (vertical)

%% Save the composition data
cd(horzcat(Dir));
save(horzcat('CompositionData',Test,'.mat'),'Composition8','Composition','Hits'...
    ,'Picturecount','Efficiency_hits','IsEmpty','Centroid','Area','data'...
    ,'Centroid','AbsIntensity','AbsMaxInt','MeanIntensity','MinIntensity'...
    ,'NumPhotos','MaxIntensity');
imwrite(Composition8,horzcat('composition',Test,'.jpg'));

%% Plot composition data

figure(1);
surf(Composition8);
title(horzcat('Surface plot of laser dots in ',Test,' Max:',num2str(MaxIntensity)...
    ,' Min:',num2str(MinIntensity),' Mean:',num2str(MeanIntensity)));
axis('equal');
axis('tight');
shading('interp');
colormap('hot');
colorbar
view(2);

%% plot sideways

figure(2);
surf(Composition8)
shading('interp');
colormap('hot');
colorbar;
view(0,0);
title(horzcat('Surface plot of laser dots in ',Test,' Max:',num2str(MaxIntensity)...
    ,' Min:',num2str(MinIntensity),' Mean:',num2str(MeanIntensity)));

%% Plot countours
%figure
%axis equal
%contourf(Composition);
%colormap cool
%colorbar('location','southoutside')